==> Botania-1.19.2-438-FABRIC-20230214.034242-11-sources.jar: vazkii/botania/common/item/CacophoniumItem.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.item;

import com.google.common.annotations.VisibleForTesting;
import net.minecraft.class_124;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1271;
import net.minecraft.class_1308;
import net.minecraft.class_1309;
import net.minecraft.class_151;
import net.minecraft.class_1548;
import net.minecraft.class_1621;
import net.minecraft.class_1657;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1836;
import net.minecraft.class_1838;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2378;
import net.minecraft.class_2428;
import net.minecraft.class_2561;
import net.minecraft.class_2960;
import net.minecraft.class_3414;
import net.minecraft.class_3417;
import net.minecraft.class_3419;
import net.minecraft.class_5328;
import net.minecraft.world.item.*;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import vazkii.botania.common.block.BotaniaBlocks;
import vazkii.botania.common.block.block_entity.CacophoniumBlockEntity;
import vazkii.botania.common.handler.BotaniaSounds;
import vazkii.botania.common.helper.ItemNBTHelper;
import vazkii.botania.mixin.MobAccessor;

import java.util.List;

public class CacophoniumItem extends class_1792 {

	private static final String TAG_SOUND = "sound";
	private static final String TAG_SOUND_NAME = "soundName";

	public CacophoniumItem(class_1793 props) {
		super(props);
	}

	@Override
	public class_1269 method_7847(class_1799 stack, class_1657 player, class_1309 entity, class_1268 hand) {
		if (entity instanceof class_1308 living) {
			class_3414 sound = null;

			if (living instanceof class_1548) {
				sound = class_3417.field_15057;
			} else if (living instanceof class_1621 slime) {
				sound = slime.method_7157() ? class_3417.field_15148 : class_3417.field_15095;
			} else {
				sound = ((MobAccessor) living).botania_getAmbientSound();
			}

			if (sound != null) {
				if (!player.field_6002.field_9236) {
					ItemNBTHelper.setString(stack, TAG_SOUND, class_2378.field_11156.method_10221(sound).toString());
					ItemNBTHelper.setString(stack, TAG_SOUND_NAME, entity.method_5864().method_5882());
					player.method_6122(hand, stack);
				}

				return class_1269.method_29236(player.field_6002.field_9236);
			}
		}

		return class_1269.field_5811;
	}

	@NotNull
	@Override
	public class_1269 method_7884(class_1838 ctx) {
		class_1799 stack = ctx.method_8041();
		if (getSound(stack) != null) {
			class_1937 world = ctx.method_8045();
			class_2338 pos = ctx.method_8037();

			class_2248 block = world.method_8320(pos).method_26204();
			if (block instanceof class_2428) {
				if (!world.method_8608()) {
					world.method_8501(pos, BotaniaBlocks.cacophonium.method_9564());
					((CacophoniumBlockEntity) world.method_8321(pos)).stack = stack.method_7972();
					stack.method_7934(1);
				}
				return class_1269.method_29236(world.method_8608());
			}
		}

		return class_1269.field_5811;
	}

	@Override
	public void method_7851(class_1799 stack, class_1937 world, List<class_2561> list, class_1836 flags) {
		if (isDOIT(stack)) {
			list.add(class_2561.method_43471("botaniamisc.justDoIt").method_27692(class_124.field_1080));
		} else if (getSound(stack) != null) {
			list.add(class_2561.method_43471(ItemNBTHelper.getString(stack, TAG_SOUND_NAME, "")).method_27692(class_124.field_1080));
		}
	}

	@Override
	public int method_7881(class_1799 stack) {
		return 72000;
	}

	@NotNull
	@Override
	public class_1271<class_1799> method_7836(class_1937 world, class_1657 player, @NotNull class_1268 hand) {
		class_1799 stack = player.method_5998(hand);
		if (getSound(stack) != null) {
			return class_5328.method_29282(world, player, hand);
		}
		return class_1271.method_22430(stack);
	}

	@Override
	public void method_7852(class_1937 world, @NotNull class_1309 living, @NotNull class_1799 stack, int count) {
		if (!world.field_9236 && count % (isDOIT(stack) ? 20 : 6) == 0) {
			playSound(living.field_6002, stack, living.method_23317(), living.method_23318(), living.method_23321(), living.method_5634(), 0.9F);
		}
	}

	public static void playSound(class_1937 world, class_1799 stack, double x, double y, double z, class_3419 category, float volume) {
		if (stack.method_7960()) {
			return;
		}

		class_3414 sound = getSound(stack);

		if (sound != null) {
			world.method_43128(null, x, y, z, sound, category, volume, sound == BotaniaSounds.doit ? 1F : (world.field_9229.method_43057() - world.field_9229.method_43057()) * 0.2F + 1.0F);
		}
	}

	@Nullable
	@VisibleForTesting
	public static class_3414 getSound(class_1799 stack) {
		if (isDOIT(stack)) {
			return BotaniaSounds.doit;
		} else {
			try {
				return class_2378.field_11156.method_17966(new class_2960(ItemNBTHelper.getString(stack, TAG_SOUND, ""))).orElse(null);
			} catch (class_151 ex) {
				return null;
			}
		}
	}

	private static boolean isDOIT(class_1799 stack) {
		return !stack.method_7960() && stack.method_7964().getString().equalsIgnoreCase("shia labeouf");
	}
}

==> Botania-1.20.1-451-FABRIC-20251214.151426-36-sources.jar: vazkii/botania/common/item/CacophoniumItem.java <==
/*
 * This class is distributed as part of the Botania Mod.
 * Get the Source Code in github:
 * https://github.com/Vazkii/Botania
 *
 * Botania is Open Source and distributed under the
 * Botania License: http://botaniamod.net/license.php
 */
package vazkii.botania.common.item;

import com.google.common.annotations.VisibleForTesting;
import net.minecraft.class_124;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1271;
import net.minecraft.class_1308;
import net.minecraft.class_1309;
import net.minecraft.class_151;
import net.minecraft.class_1548;
import net.minecraft.class_1621;
import net.minecraft.class_1657;
import net.minecraft.class_1792;
import net.minecraft.class_1799;
import net.minecraft.class_1836;
import net.minecraft.class_1838;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2428;
import net.minecraft.class_2561;
import net.minecraft.class_2960;
import net.minecraft.class_3414;
import net.minecraft.class_3417;
import net.minecraft.class_3419;
import net.minecraft.class_5328;
import net.minecraft.class_5712;
import net.minecraft.class_7923;
import net.minecraft.world.item.*;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import vazkii.botania.common.block.BotaniaBlocks;
import vazkii.botania.common.block.block_entity.CacophoniumBlockEntity;
import vazkii.botania.common.handler.BotaniaSounds;
import vazkii.botania.common.helper.ItemNBTHelper;
import vazkii.botania.mixin.MobAccessor;

import java.util.List;

public class CacophoniumItem extends class_1792 {

	private static final String TAG_SOUND = "sound";
	private static final String TAG_SOUND_NAME = "soundName";

	public CacophoniumItem(class_1793 props) {
		super(props);
	}

	@Override
	public class_1269 method_7847(class_1799 stack, class_1657 player, class_1309 entity, class_1268 hand) {
		if (entity instanceof class_1308 living) {
			class_3414 sound = null;

			if (living instanceof class_1548) {
				sound = class_3417.field_15057;
			} else if (living instanceof class_1621 slime) {
				sound = slime.method_7157() ? class_3417.field_15148 : class_3417.field_15095;
			} else {
				sound = ((MobAccessor) living).botania_getAmbientSound();
			}

			if (sound != null) {
				if (!player.method_37908().field_9236) {
					ItemNBTHelper.setString(stack, TAG_SOUND, class_7923.field_41172.method_10221(sound).toString());
					ItemNBTHelper.setString(stack, TAG_SOUND_NAME, entity.method_5864().method_5882());
					player.method_6122(hand, stack);
				}

				return class_1269.method_29236(player.method_37908().field_9236);
			}
		}

		return class_1269.field_5811;
	}

	@NotNull
	@Override
	public class_1269 method_7884(class_1838 ctx) {
		class_1799 stack = ctx.method_8041();
		if (getSound(stack) != null) {
			class_1937 world = ctx.method_8045();
			class_2338 pos = ctx.method_8037();

			class_2248 block = world.method_8320(pos).method_26204();
			if (block instanceof class_2428) {
				if (!world.method_8608()) {
					world.method_8501(pos, BotaniaBlocks.cacophonium.method_9564());
					((CacophoniumBlockEntity) world.method_8321(pos)).stack = stack.method_7972();
					stack.method_7934(1);
				}
				return class_1269.method_29236(world.method_8608());
			}
		}

		return class_1269.field_5811;
	}

	@Override
	public void method_7851(class_1799 stack, class_1937 world, List<class_2561> list, class_1836 flags) {
		if (isDOIT(stack)) {
			list.add(class_2561.method_43471("botaniamisc.justDoIt").method_27692(class_124.field_1080));
		} else if (getSound(stack) != null) {
			list.add(class_2561.method_43471(ItemNBTHelper.getString(stack, TAG_SOUND_NAME, "")).method_27692(class_124.field_1080));
		}
	}

	@Override
	public int method_7881(class_1799 stack) {
		return 72000;
	}

	@NotNull
	@Override
	public class_1271<class_1799> method_7836(class_1937 world, class_1657 player, @NotNull class_1268 hand) {
		class_1799 stack = player.method_5998(hand);
		if (getSound(stack) != null) {
			return class_5328.method_29282(world, player, hand);
		}
		return class_1271.method_22430(stack);
	}

	@Override
	public void method_7852(class_1937 world, @NotNull class_1309 living, @NotNull class_1799 stack, int count) {
		if (!world.field_9236 && count % (isDOIT(stack) ? 20 : 6) == 0) {
			playSound(living.method_37908(), stack, living.method_23317(), living.method_23318(), living.method_23321(), living.method_5634(), 0.9F);
			living.method_32876(class_5712.field_39415);
		}
	}

	public static void playSound(class_1937 world, class_1799 stack, double x, double y, double z, class_3419 category, float volume) {
		if (stack.method_7960()) {
			return;
		}

		class_3414 sound = getSound(stack);

		if (sound != null) {
			world.method_43128(null, x, y, z, sound, category, volume, sound == BotaniaSounds.doit ? 1F : (world.field_9229.method_43057() - world.field_9229.method_43057()) * 0.2F + 1.0F);
		}
	}

	@Nullable
	@VisibleForTesting
	public static class_3414 getSound(class_1799 stack) {
		if (isDOIT(stack)) {
			return BotaniaSounds.doit;
		} else {
			try {
				return class_7923.field_41172.method_10223(new class_2960(ItemNBTHelper.getString(stack, TAG_SOUND, "")));
			} catch (class_151 ex) {
				return null;
			}
		}
	}

	private static boolean isDOIT(class_1799 stack) {
		return !stack.method_7960() && stack.method_7964().getString().equalsIgnoreCase("shia labeouf");
	}
}
